#####
# Script for extracting adult vital rates (survival and flowering probabilities)
# from mark recapture models for use in matrix models in Nordstrom et al. 2021
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####

# Load packages
library(dplyr)
library(tidyr)

# Load in data
# (note: for purposes of this script, load in original data from analysis)
load('03_vital_rate_posteriors/MSMR_raw_output_original.RData')

ls()

# Wrapper functon for extracting vital rates of interest
extract.pars = function(stan.mod, sample.inds) {
  stan.mod %>%
    as.data.frame() %>%
    # Remove detection probabilities
    select(-contains('p')) %>%
    # Trial in this case means posterior estimate in chain
    mutate(trial = 1:4000) %>%
    gather(param, estimate, -trial) %>%
    # Filter out only a certain number of estimates
    filter(trial %in% sample.inds) %>%
    # Add info for each estimate
    mutate(vital = ifelse(grepl('s', param), 's', 'f'),                  # denotes survival or flowering
           stage = ifelse(grepl('\\[1', param), 'b', 'f'),               # denotes whether plant is basal or flowering
           year = gsub('[sf]\\[\\d\\,|\\]', '', param) %>% as.numeric(), # extract year (and below)
           year = year + 1998,
           rate = paste(vital, stage, sep = '_'),                        # assign vital rate
           trial = dense_rank(trial))                                    # reorder trial info
}

set.seed(651929)

# Extract vital rates
adult.ests = mget(grep('\\_mod', ls(), value = TRUE)) %>%
  lapply(extract.pars, sample.inds = sample(4000, 100, replace = FALSE)) %>%
  do.call(what = rbind) %>%
  filter(year %in% 2000:2014) # get only years of interest (2000 - 2014; pre-2000 is essentially burn-in)
# Format sites
adult.ests$site = grep('\\_mod', ls(), value = TRUE) %>%
  gsub(pattern = '\\_mod', replacement = '') %>%
  rep(each = 15 * 4 * 100)

head(adult.ests)
tail(adult.ests)
table(adult.ests$site)
table(adult.ests$year)

# We were missing data for 2008 YOH and 2009 RRXDC
# For these sites, average out previous and next year's estimates
adult.ests = adult.ests %>%
  group_by(rate, site, trial) %>%
  mutate(estimate = ifelse(site %in% 'yoh' & year %in% 2008, 
                             mean(estimate[year %in% c(2007, 2009)]), estimate),
         estimate = ifelse(site %in% 'rrxdc' & year %in% 2009,
                             mean(estimate[year %in% c(2008, 2010)]), estimate)) %>%
  ungroup()

# Write results to csv file
write.csv(adult.ests, row.names = FALSE, file = '03_vital_rate_posteriors/adult_rates_bayesian.csv')

# # Aux code for plotting estimates
# for (s in grep('\\_mod', ls(), value = TRUE) %>% gsub(pattern = '\\_mod', replacement = '')) {
#   g = adult.ests %>%
#     filter(site %in% s) %>%
#     ggplot() +
#     geom_violin(aes(x = as.factor(year), y = estimate)) +
#     facet_wrap(~ rate) +
#     theme(axis.text.x = element_text(angle = 90)) +
#     ggtitle(s) +
#     xlab("year")
#   
#   # ggsave(paste0('FILENAME', s, '.pdf'), g)
# }
