# A script for modeling the flowering of juvenile plants.
# This script also produces juvenile survival estimates for
# matrix models used in Nordstrom et al., 2021.
# Contact: scottwatsonnordstrom [at] gmail [dot] com

##### Load in necessary packages
library(dplyr)
library(tidyr)
library(brms) 

# NOTE: models were originally fit with Version 2.7.0 in  Version 2.12.0

rm(list = ls())

###### Load in data

# Note: this is the same as the high juvenile survival dataset
# (data from Wagenius et al., 2012) with records up to 2014
j.flw.data = read.csv('https://www.dropbox.com/s/5ed5hmi9k6fyjxa/high_juvenile_survival_census_for_matrix_models.csv?raw=1')
# We are only interested in plants at reproductive age to filter to only include
# zones with plants ages 8+.
j.flw.data = j.flw.data[j.flw.data$age3 %in% '8+',]

head(j.flw.data)
# FIELDS:
# zone - circle into which seeds were scattered; this is the experimental unit
# of replication; integer coded
# unitID - intermediate blocking variable; zones were blocked into units
# burn treatments were applied at the unit level w/ the exception of one
# uncontrolled burn which burned multiple units; integer coded
# plot - site at which units are established; integer coded
# year - year of observation
# count - number of surviving, non-flowering plants (note: not used in this
# analysis)
# burn - was zone urned in the given year?
# planted - year seeds were scattered
# vegm - vegetation type (this analysis was performed entirely on oldfields)
# mean.temp - mean teperature in ~12 months prior to observation (mean-centered)
# max.temp - highest temperature in ~12 months prior to obs. (mean-centered)
# total.precip - total precipitaton over prior 12 months (mean-centered)
# age1, age2, age3, age4, age5 - age categories for plants in zone (note: not
# used in this analysis b/c all plants modeled were of flowering age)
# prior - number of plants in zone prior to observation
# (used in models as the number of trials for binomial response)
# burn1, burn2 - burns one or two years previous to observation (not used in
# this analysis - see note below)
# obs - a unique identifier for each observation (was used in preliminary models)

##### Modeling

# Here, model the number of flowering plants in a zone as a binomial random variable.
# Use plot as a random effect
# Preliminary modeling (not shown here, available upon request) suggested that
# additional spatial structure (unit, plot nested within plot) did not improve
# model quality.
# Prior fitting of frequentist models also suggested that burn1 and burn2 (i.e.,
# burning in prior years to observation) did not influence burn survival. As
# such we did not perform model selection with these variables.
# Additionally, initial modeling demonstrated poor convergence (likely due to
# high volumes of zeros in this dataset); this was fixed by setting adapt_delta
# to 0.99 in all models.

### First, model using maximum temperature to capture annual variation.

# set priors
pri = c(set_prior("normal(0, 10)", class = "b", coef = "burnTRUE"),
        set_prior("normal(0, 10)", class = "b", coef = "max.temp"))

# fit model with maximum temperature (instead of year)
a.max = brm(flpl | trials(prior) ~ burn + max.temp + (1 | plot),
            data = j.flw.data,
            family = binomial(),
            cores = 4,
            prior = pri,
            seed = 1775769,
            control = list(adapt_delta = 0.99))

### A model using mean temperature for annual variaiton

# set priors
pri = c(set_prior("normal(0, 10)", class = "b", coef = "burnTRUE"),
        set_prior("normal(0, 10)", class = "b", coef = "mean.temp"))

# fit model with mean temperature (instead of year)
a.mnt = brm(flpl | trials(prior) ~ burn + mean.temp + (1 | plot),
            data = j.flw.data,
            family = binomial(),
            cores = 4,
            prior = pri,
            seed = 1165653,
            control = list(adapt_delta = 0.99))

pri = c(set_prior("normal(0, 10)", class = "b", coef = "burnTRUE"),
        set_prior("normal(0, 10)", class = "b", coef = "total.precip"))

a.prc = brm(flpl | trials(prior) ~ burn + total.precip + (1 | plot),
            data = j.flw.data,
            family = binomial(),
            cores = 4,
            prior = pri,
            seed = 1165653,
            control = list(adapt_delta = 0.99))

pri = c(set_prior("normal(0, 10)", class = "b", coef = "burnTRUE"))

a.noc = brm(flpl | trials(prior) ~ burn + as.factor(year) + (1 | plot),
            data = j.flw.data,
            family = binomial(),
            cores = 4,
            prior = pri,
            seed = 1165653,
            control = list(adapt_delta = 0.99))

loo.all = loo(a.max, a.mnt, a.prc, a.noc, reloo = TRUE)

loo.all

a.mnt

# Family: binomial 
# Links: mu = logit 
# Formula: flpl | trials(prior) ~ burn + mean.temp + (1 | plot) 
# Data: j.flw.data (Number of observations: 717) 
# Samples: 4 chains, each with iter = 2000; warmup = 1000; thin = 1;
# total post-warmup samples = 4000
# 
# Group-Level Effects: 
# ~plot (Number of levels: 5) 
# Estimate Est.Error l-95% CI u-95% CI Eff.Sample Rhat
# sd(Intercept)     2.67      2.04     0.76     7.75        850 1.00
# 
# Population-Level Effects: 
#           Estimate Est.Error l-95% CI u-95% CI Eff.Sample Rhat
# Intercept    -4.98      1.49    -8.31    -2.22        891 1.00
# burnTRUE      1.01      0.32     0.37     1.63       2967 1.00
# mean.temp    -0.12      0.06    -0.25    -0.01       2816 1.00
# 
# Samples were drawn using sampling(NUTS). For each parameter, Eff.Sample 
# is a crude measure of effective sample size, and Rhat is the potential 
# scale reduction factor on split chains (at convergence, Rhat = 1).

ppp = posterior_predict(a.mnt, nsamples = 100)

plot(j.flw.data$count, apply(ppp, 2, mean))
bayesplot::ppc_scatter_avg(j.flw.data$count, ppp)

plot(jitter(j.flw.data$total.precip), fitted.values(a.mnt, type = 'linear') %>% as.data.frame() %>% select(Estimate) %>% unlist())

ranef(a.mnt) # mean zone is 3

nd = expand.grid(year = 2000:2014, burn = c(TRUE, FALSE), plot = 3, prior = 1) %>%
  merge(y = read.csv('annual_climate.csv') %>% select(season, mean.temp) %>% rename(year = season))

set.seed(9119901)
juv_flower = posterior_predict(a.mnt, newdata = nd, type = 'response') %>% 
  (function(x) replicate(100, apply(x, 2, sample, replace = TRUE))) %>%
  apply(2:3, mean) %>%
  cbind(nd, .) %>%
  as.data.frame() %>%
  select(-c(plot, prior, mean.temp)) %>%
  gather(trial, estimate, -c(year, burn)) %>%
  mutate(trial = as.numeric(trial))
  
juv_flower %>%
  ggplot() +
  geom_histogram(aes(x = estimate, fill = burn), binwidth = 0.001) +
  scale_fill_manual(values = c("black", "red"))

juv_flower %>%
  ggplot() +
  geom_histogram(aes(x = estimate, fill = burn), binwidth = 0.001) +
  scale_fill_manual(values = c("black", "red")) +
  facet_wrap(~ year)

head(juv_flower)
with(juv_flower, table(year, burn))
with(juv_flower, table(year, trial)) %>% unique()

write.csv(juv_flower, row.names = FALSE, file = 'juv_flower_bayesian.csv')

