# Script for modeling juvenile survival, quantifying the effects of burning on
# juvenile survival, and producing estimates for matrix models, 
# for Nordstrom et al., 2021.
# Contact: scottwatsonnordstrom [at] gmail [dot] com

##### Load in packages.

library(ggplot2)
library(dplyr)
library(tidyr)
library(brms) # models originally fit in version 2.7.0

##### Clear namespace
rm(list = ls())

##### Load in LOW juvenile survival census.
low.juv.surv = read.csv('https://www.dropbox.com/s/0jm4hojh32cqg34/low_juvenile_survival_census_for_matrix_models.csv?raw=1')

head(low.juv.surv)
# Fields:
# year - year of observation
# focalCd - unique identifier for the _maternal parent_ of the seedling
# slingCd - unique identifier for the _seedling_
# site - population
# ld - was plant alive in observation (ld is boolean - 0 or 1)
# total.precip - total precipitation in prior 12 months
# mean.temp - mean monthly temperature in prior 12 months
# max.temp - maximum temperature over last 12 months
# burn, burn1 - burn status
# age, age2, age3, age4, age5 - age categories
# obs - unique identifier for observation number


##### Fit models

### First, determine best climatic predictor

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age32P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'max.temp'))

# Model with maximum temperature
br.maxt = brm(ld ~ age3 + max.temp + (1 | site / focalCd),
              family = bernoulli(),
              data = low.juv.surv,
              seed = 339970,
              prior = pri,
              cores = 4)
# loo.maxt = loo(br.maxt, reloo = TRUE)

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age32P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Model with mean temperature
br.mean = brm(ld ~ age3 + mean.temp + (1 | site / focalCd),
              family = bernoulli(),
              data = low.juv.surv,
              seed = 128348,
              prior = pri,
              cores = 4)
# loo.mean = loo(br.mean, reloo = TRUE)

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age32P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'))

# Model featuring total precipitatoin
br.prec = brm(ld ~ age3 + total.precip + (1 | site / focalCd),
              family = bernoulli(),
              data = low.juv.surv,
              seed = 128348,
              prior = pri,
              cores = 4)
# loo.prec = loo(br.prec, reloo = TRUE)

# Set Bayesian prior
pri = set_prior('normal(0, 5)', class = 'b', coef = 'age32P')

# Model with year fixed effect
br2 = brm(ld ~ age3 + as.factor(year) + (1 | site / focalCd),
          family = bernoulli(),
          data = low.juv.surv,
          seed = 1940234,
          prior = pri,
          cores = 4)

# Leave one out cross-validation of all models.
(loo.p = loo(br2, br.prec, br.mean, br.maxt, reloo = TRUE))
# Best model appears to be br2, with year fixed effect.

# Posterior predictive check on br2
ppp = posterior_predict(br2, nsamples = 100, seed = 148566)

# Look at mean posterior estimates
bayesplot::ppc_stat(low.juv.surv$ld, ppp, stat = 'mean')
# Looks good to me.

# Look at overdispersion
bayesplot::ppc_stat(low.juv.surv$ld, ppp, stat = function(x) mean(x == 0))
# Also looks within range to me.

# Object for random effects (determining mean site/year)
rfx = ranef(br2)

# Determining the site closest to the mean (i.e. intercept closest to zero in magnitude)
rfx$site %>% 
  data.frame() %>%
  tibble::rownames_to_column() %>%
  rename(est = Estimate.Intercept) %>%
  mutate(med = min(abs(est)))
# Closest site to mean is ri

# Determining mean focal plant for each site
# Used for generating posterior predictions used  to parameterize matrices
med.focals = rfx$`site:focalCd` %>%
  data.frame() %>%
  tibble::rownames_to_column() %>%
  rename(id = rowname,
         est = Estimate.Intercept) %>%
  separate(id, c("site", "focalCd"), sep = '\\_') %>%
  group_by(site) %>%
  mutate(med = min(abs(est))) %>%
  filter(med == abs(est)) %>%
  distinct(site, .keep_all = TRUE)

# Find median year (for out-of-model predictions)
# (In retrospect - should have chosen the year closest to zero as done above -
# oh well, effects on the final estimates will be extremely marginal)
fixef(br2) %>% (function(x) x[-(1:2),]) %>% as.data.frame() %>% select(Estimate) %>% 
  unlist() %>% (function(x) median(c(0, x)))
# Original script had the medan year as 2013 
# (although with the chnge in packages it now looks like it would be 2010 or 2015)

# Form backbone for model fits
# First have all site, year, age combinations
# set site to riley if not in original model, set year to 2015 if out of model
# then add site median
js = expand.grid(site.orig = c("eth", "lce", "lcw", "nrrx", "nwlf", "nnwlf", "rrxdc", "sgc", "yoh"),
                 year.orig = 2000:2014,
                 age3 = c("1", "2+")) %>%
  mutate(site = ifelse(site.orig %in% low.juv.surv$site, as.character(site.orig), 'ri'),
         year = ifelse(year.orig %in% 2008:2017, year.orig, 2013)) %>%
  merge(y = med.focals[,c("focalCd", "site")], by = 'site', all.x = TRUE)

# Generate model posterior predictions using backbone.
# Then, bootstrap out 100 means from these posteriors.
# Finally, reformat (take out extraneous columns)
set.seed(8002704)
br2.posterior = posterior_predict(br2, newdata = js, type = 'response') %>%
  (function(x) replicate(100, apply(x, 2, sample, replace = TRUE))) %>%
  apply(2:3, mean) %>%
  cbind(js) %>%
  gather(key = trial, value = estimate, as.character(1:100)) %>%
  select(-c(site, focalCd, year)) %>%
  rename(site = site.orig,
         year = year.orig)


# A plot, looking at distributions of posteriors
br2.posterior %>%
  ggplot() +
  geom_histogram(aes(x = estimate, fill = site), binwidth = 0.01) +
  scale_fill_manual(values = RColorBrewer::brewer.pal(9, 'Spectral')) +
  facet_wrap(~ age3)

# Export low juvenile survival posterior estimates for matrix models.
# NOTE: this repository has the original output of this model
# Changes to brms since then have produced slightly different estimates.
write.csv(br2.posterior, row.names = FALSE, file = '03_vital_rate_posteriors/juv_surv_bayesian_lojuv.csv')


##### Clear namespace
rm(list = ls())

##### Fit models of high juvenile survival.

high.juv.surv = read.csv('https://www.dropbox.com/s/5ed5hmi9k6fyjxa/high_juvenile_survival_census_for_matrix_models.csv?raw=1')

# Fields
# zone, unitID, plot - nested spatial blocks (n.b., plot is
# NOT one of the same remnant sites used in other datasets).
# year - year of observation
# count - number of seedlings observed alive
# prior - count in PREVIOUS year (used binomail response)
# flpl - number of flowering plants observed in plot
# vegm - vegetation type (subsetted to only olfields)
# mean.temp - mean temperature over prior 12 years
# max.temp - maximum temperature over prior year
# total.precip - total precipitaton over prior 12 months
# age1, age2, age3, age4, age5 - age of plants in zone (binned 
# into various categories)
# burn1, burn2 - burn status
# obs - unique identifier for observation

### First, try various annual variation covariates.

# NOTE: original models had a binomial link function (not shown),
# but these models featured overdispersion.
# As such, fit new models with a beta-binomial link function.

# We fit a set of models (not shown) with different juvenile age structures
# and different year/climate covariates.
# We found (not shown) that the best age structure was 0, 1, 2+ years
# and that the best climatic predictors were mean temperature
# and cumulative precipitation.
# Code for these models available upon request:
# email scottwatsonnordstrom [at] gmail [dot] com

# Got this code from somewhere, likely on the Rstan support board
# or other features. Unfortunately the link has been lost;
# can't say much other than that we did not write this code
# to implement this beta binomial link.

# Setting up beta binomial link.
beta_binomial2 = custom_family(
  "beta_binomial2", dpars = c("mu", "phi"),
  links = c("logit", "log"), lb = c(NA, 0),
  type = "int", vars = "prevs[n]"
)

stan_funs = "
  real beta_binomial2_lpmf(int y, real mu, real phi, int T) {
return beta_binomial_lpmf(y | T, mu * phi, (1 - mu) * phi);
}
int beta_binomial2_rng(real mu, real phi, int T) {
return beta_binomial_rng(T, mu * phi, (1 - mu) * phi);
}
"

stanvars = stanvar(scode = stan_funs, block = "functions") + stanvar(as.integer(high.juv.surv$prior), name = "prevs")

# These functions needed for posterior prediction and leave one out cross validation.
log_lik_beta_binomial2 = function(i, draws) {
  mu <- draws$dpars$mu[, i]
  phi <- draws$dpars$phi
  N <- draws$data$prevs[i]
  y <- draws$data$Y[i]
  beta_binomial2_lpmf(y, mu, phi, N)
}

predict_beta_binomial2 = function(i, draws, ...) {
  mu <- draws$dpars$mu[, i]
  phi <- draws$dpars$phi
  N <- draws$data$trials[i]
  beta_binomial2_rng(mu, phi, N)
}

# Now, fit candidate models.

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Fit model with burn and burn1
b1pm.a4 = brm(count | trials(prior) ~ age4 + burn + burn1 + 
              total.precip + mean.temp + (1 | plot / unitID / zone),
              family = beta_binomial2, stanvars = stanvars, 
              data = high.juv.surv, prior = pri,
              seed = 55949, cores = 4,
              control = list(adapt_delta = 0.99))

b1pm.a4

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Fit model with only burn (i.e., no effect of burning in PRIOR year)
b0pm.a4 = brm(count | trials(prior) ~ age4 + burn +
              total.precip + mean.temp + (1 | plot / unitID / zone),
              family = beta_binomial2, stanvars = stanvars, 
              data = high.juv.surv, prior = pri,
              seed = 22808, cores = 4,
              control = list(adapt_delta = 0.99))


# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'))

# Fit model with burn2
b.y.a4 = brm(count | trials(prior) ~ age4 + burn + burn1 + burn2 +
             as.factor(year) + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 808905, cores = 4,
             control = list(adapt_delta = 0.99))
# Note: 2 divergent pairs

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'))

# Fit model with only precipitation for annual variation (no mean temp)
b.p.a4 = brm(count | trials(prior) ~ age4 + burn + burn1 + burn2 +
             total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 293410, cores = 4,             
             control = list(adapt_delta = 0.99))

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Fit model with only mean temperature (no precipitatoin)
b.m.a4 = brm(count | trials(prior) ~ age4 + burn + burn1 + burn2 +
             mean.temp + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 692694, cores = 4,
             control = list(adapt_delta = 0.99))

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'max.temp'))

# Fit model with maximum temperature (just to make sure this
# won't outperform other models)
b.x.a4 = brm(count | trials(prior) ~ age4 + burn + burn1 + burn2 +
             max.temp + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 115964, cores = 4,
             control = list(adapt_delta = 0.99))

# Test leave one out cross validation.
loo.x = loo(b.x.a4) # 2470.3
loo.m = loo(b.m.a4) # 2461.4
loo.p = loo(b.p.a4) # 2434.7
loo.y = loo(b.y.a4) # 2360.8

# Age works best, out of all of these, but is potentially confounding with year or cohort.
# Of the remaining, precipitaton looks fine.

# Now, test some different age classifications.

# Set Bayesian prior.
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Try age4 (ages 0, 1, 2+) (coded here as 1, 2, 3+)
bmp.a4 = brm(count | trials(prior) ~ age4 + burn + burn1 + burn2 +
             mean.temp + total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 155097, cores = 4,             
             control = list(adapt_delta = 0.99))
loo.mp = loo(bmp.a4) # 2428.0

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age32M7'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age38P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Try age3 (1, 2-7, age 8+, differentiating reproductive-age plants
# from non-reproductive age plants)
bmp.a3 = brm(count | trials(prior) ~ age3 + burn + burn1 + burn2 +
             mean.temp + total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 26111211, cores = 4,             
             control = list(adapt_delta = 0.99))
loo.a3 = loo(bmp.a3) # 2426.7

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age52'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age53M7'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age58P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn2TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Try model with (1, 2, 3-7, 8+)
bmp.a5 = brm(count | trials(prior) ~ age5 + burn + burn1 + burn2 +
             mean.temp + total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 717173, cores = 4,             
             control = list(adapt_delta = 0.99))
# two divergences
loo.a5 = loo(bmp.a5)# 2422.0

# Of these, bst age structure is age4 (0 year old, 1 year old, 2+ year old)

# Now, perform model selection, dropping terms.

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burn1TRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Drop burn2
bmp1a4 = brm(count | trials(prior) ~ age4 + burn + burn1 +
             mean.temp + total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 4265672, cores = 4,             
             control = list(adapt_delta = 0.99))
loo.1mp = loo(bmp1a4) # 2428.9

compare_ic(loo.mp, loo.a5)

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Drop burn1
bmp0a4 = brm(count | trials(prior) ~ age4 + burn +
             mean.temp + total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 3285649, cores = 4,             
             control = list(adapt_delta = 0.99))
loo.0mp = loo(bmp0a4) # 2427.0

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Drop burn
bmpna4 = brm(count | trials(prior) ~ age4 + mean.temp + total.precip 
             + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 9292496, cores = 4,             
             control = list(adapt_delta = 0.99))

loo.nmp = loo(bmpna4) # 2425.2 

# Set Bayesian prior
pri = c(set_prior('normal(0, 5)', class = 'b', coef = 'age42'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P'),
        set_prior('normal(0, 5)', class = 'b', coef = 'burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age42:burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'age43P:burnTRUE'),
        set_prior('normal(0, 5)', class = 'b', coef = 'total.precip'),
        set_prior('normal(0, 5)', class = 'b', coef = 'mean.temp'))

# Fit model with interaction between age and burn.
bmpxa4 = brm(count | trials(prior) ~ age4 * burn +
             mean.temp + total.precip + (1 | plot / unitID / zone),
             family = beta_binomial2, stanvars = stanvars, 
             data = high.juv.surv, prior = pri,
             seed = 30100, cores = 4,             
             control = list(adapt_delta = 0.99))
loo.xmp = loo(bmpxa4)

compare_ic(loo.nmp, loo.xmp, loo.0mp, loo.1mp, loo.mp)

#                   LOOIC    SE
# bmpna4          2425.22 85.18
# bmpxa4          2405.02 85.12
# bmp0a4          2426.97 85.19
# bmp1a4          2428.95 85.32
# bmp1a4          2428.95 85.32
# bmpna4 - bmpxa4   20.20 10.46
# bmpna4 - bmp0a4   -1.75  1.31
# bmpna4 - bmp1a4   -3.73  1.85
# bmpna4 - bmp1a4   -3.73  1.85
# bmpxa4 - bmp0a4  -21.95 10.54
# bmpxa4 - bmp1a4  -23.93 10.58
# bmpxa4 - bmp1a4  -23.93 10.58
# bmp0a4 - bmp1a4   -1.98  1.44
# bmp0a4 - bmp1a4   -1.98  1.44
# bmp1a4 - bmp1a4    0.00  0.00

# Of all of these, model with interaction term is by far the best.

# These lines are NEEDED to generate posterior checks

log_lik_beta_binomial2 = function(i, draws) {
  mu <- draws$dpars$mu[, i]
  phi <- draws$dpars$phi
  N <- draws$data$prevs[i]
  y <- draws$data$Y[i]
  beta_binomial2_lpmf(y, mu, phi, N)
}

predict_beta_binomial2 = function(i, draws, ...) {
  mu <- draws$dpars$mu[, i]
  phi <- draws$dpars$phi
  N <- draws$data$trials[i]
  beta_binomial2_rng(mu, phi, N)
}

expose_functions(bmpxa4, vectorize = TRUE)

pp_check(bmpxa4)
# There is some slight overfitting but not enough to be worried about.

ppp = posterior_predict(bmpxa4, seed = 210718, nsamples = 100)

bayesplot::ppc_stat(high.juv.surv$count, ppp, stat = 'mean') # Good
bayesplot::ppc_stat(high.juv.surv$count, ppp, stat = function(x) mean(x/ high.juv.surv$prior)) # This looks pretty good
bayesplot::ppc_stat(high.juv.surv$count, ppp, stat = function(x) mean(x == 0)) 
# This is slightly underdispersed, but not by a whole lot 
# - differences between 2% and 3% are small
mean(apply(ppp, 1, function(x) mean(x == 0)) < mean(!high.juv.surv$count)) 
# 91% of these posteriod draws have fewer 0s
mean(abs(apply(ppp, 1, function(x) mean(x == 0)) - mean(!high.juv.surv$count))) 
# Average difference is 0.4%. I think this is fine.

# Here, find the mean population for site-level random effects.
merge(x = ranef(bmpxa4)$`plot:unitID:zone` %>% as.data.frame() %>%
        rename(zone.efct = Estimate.Intercept) %>% select(zone.efct) %>% 
        tibble::rownames_to_column(var = 'full.zone') %>%
        separate(full.zone, c("plot", "unitID", "zone")),
      y = ranef(bmpxa4)$`plot:unitID` %>% as.data.frame() %>%
        rename(unit.efct = Estimate.Intercept) %>% select(unit.efct) %>%
        tibble::rownames_to_column(var = 'full.unit') %>%
        separate(full.unit, c("plot", "unitID")),
      by = c("plot", "unitID")) %>%
  merge(y = ranef(bmpxa4)$plot %>% as.data.frame() %>%
          rename(plot.efct = Estimate.Intercept) %>% select(plot.efct) %>%
          tibble::rownames_to_column(var = 'plot'),
        by = "plot") %>%
  mutate(total.effect = zone.efct + unit.efct + plot.efct,
         abval.effect = abs(total.effect)) %>%
  filter(abval.effect == min(abval.effect))

# plot 5, unit 25, zone 75 is the mean plot, unit, zone combo - 
# use this to generate all posterior predictions 
# (effectively ignoring space)

# Here, generate an empty data frame for creating juvenile survival posteriors
# to put into matrix. (nd = newdata argument in posterior_predict)
nd = expand.grid(age4 = c('1', '2', '3+'), burn = c(TRUE, FALSE), year = 2000:2014) %>%
  # Merge with climate data
  merge(y = read.csv('annual_climate_2000_2017.csv') %>% select(season, total.precip, mean.temp),
        by.x = 'year', by.y = 'season') %>%
  # Necessary covariates for out-of-sample predictions
  # (prior = 1 means predict survival/death for one plant, i.e., Bernoulli trial)
  mutate(prior = 1, plot = 5, unitID = 25, zone = 75)


# Use data frame above to generate posterior predictions.
# (js = juvenile survival)
set.seed(879701)
js = posterior_predict(bmpxa4, newdata = nd, type = 'response', re.form = NA) %>%
  # bootstrap means (posterior predictions are integer-valued, this gives means)
  (function(x) replicate(100, apply(x, 2, sample, replace = TRUE))) %>%
  apply(2:3, mean) %>%
  # Add on backbone.
  cbind(nd) %>%
  gather(key = trial, value = estimate, as.character(1:100))

# Export data.
# NOTE: this repository has the original output of this model
# Changes to brms since then have produced slightly different estimates.
write.csv(js, row.names = FALSE, file = '03_vital_rate_posteriors/juv_surv_bayesian_hijuv.csv')
