# This is a script for modeling seedling recruitment.
# This was used to estimate fire's effects on seedling recruitment, and
# correspondingly, post-fire recruitment's influence on population dynamics,
# in Nordstrom et al., 2021
# Contact: scottwatsonnordstrom [at] gmail [dot] com

# Load in necessary packages.
library(ggplot2)
library(dplyr)
library(tidyr)
library(brms) # Version 2.7.0

rm(list = ls())

##### Load in dataset

recruit.data = read.csv('https://www.dropbox.com/s/znycfs1ol20ts4z/seedling_recruitment_for_matrix_models.csv?raw=1')
# IMPORTANT FIELDS:
# year (of SEEDLING CENSUS - flowering year is year prior)
# aka is unique identifier for focal (maternal) plant
# focalCd is an identifier for focal plant-year combination
# radius is radius (in cm) of search circle around plant
# hdCt is number of heads for focal plant in year of flowering
# slingCt is number of seedlings observed in circle
# burn, burn1, burn2, burn3 are codes for burn status,
# burn is observation in a burn year, burn1 is observation one year
# post-burn, etc.
# hdCt2 is head count TWO years prior to census;
# this was recommended as a way to search for a seed bank
# (if this is a significant predictor in models, then seeds likely are dormant)
# total.precip is precipitation in 12 months prior to census
# mean.temp is mean monthly temperature over same span
# max.temp is maximum monthly temperature over same span

# First, note that all observations in burn years have no seedlings.
with(recruit.data, table(burn, slingCt > 0))
# burn    FALSE TRUE
# FALSE   880  306
# TRUE     37    0

# This means that GLMMs can't properly estimate the uncertainty around
# the effect of burning (dividing by variance of zero, I presume).

# To remedy this, we'll remove all of these observations from the dataset. We'll
# make the assumption that, if there are seedlings here, they are likely being
# killed. Consequently, in matrix models, we'll set recruitment in burn years to
# be zero (with zero variance around that).

recruit.data = recruit.data[!recruit.data$burn,]

##### Fit models now

### Do this in stages. First, check to see what the annual variation should look like.
# First, fit models with year as covariate. Then, fit models with climate covariates.

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn3TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "loghdCt"),
        set_prior("normal(0, 5)", class = "b", coef = "hdCt2"))

# Fit model with year fixed-effect covariate.
mf.year = brm(slingCt ~ burn1 + burn2 + burn3 + log(hdCt) + hdCt2 + 
                        year + (1 | site),
              data = recruit.data,
              family = negbinomial(), prior = pri,
              cores = 4, seed = 2333222)

# Calculate LOO-CV for this model.  
loot = loo(mf.year, reloo = TRUE, seed = 50267)

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn3TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "max.temp"),
        set_prior("normal(0, 5)", class = "b", coef = "loghdCt"),
        set_prior("normal(0, 5)", class = "b", coef = "hdCt2"))

# Fit model with maximum temperature rather than year.
mf.maxt = brm(slingCt ~ burn1 + burn2 + burn3 + log(hdCt) + hdCt2 + 
                        max.temp + (1 | site),
              data = recruit.data,
              family = negbinomial(), prior = pri,
              cores = 4, seed = 645000)

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn3TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "mean.temp"),
        set_prior("normal(0, 5)", class = "b", coef = "loghdCt"),
        set_prior("normal(0, 5)", class = "b", coef = "hdCt2"))

# Fit model with mean temperature rather than year.
mf.mean = brm(slingCt ~ burn1 + burn2 + burn3 + log(hdCt) + hdCt2 +
                        mean.temp + (1 | site),
              data = recruit.data,
              family = negbinomial(), prior = pri,
              cores = 4, seed = 483153)

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn3TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "total.precip"),
        set_prior("normal(0, 5)", class = "b", coef = "loghdCt"),
        set_prior("normal(0, 5)", class = "b", coef = "hdCt2"))

# Fit model with precipitation rather than year.
mf.prec = brm(slingCt ~ burn1 + burn2 + burn3 + log(hdCt) + hdCt2 +
                        total.precip + (1 | site),
              data = recruit.data,
              family = negbinomial(), prior = pri,
              cores = 4, seed = 461889)

# Compare models using LOO-CV
loo.annvar = loo(mf.maxt, mf.mean, mf.prec, reloo = TRUE, seed = 921239)

loot # looic is 2416.1
# Computed from 4000 by 1186 log-likelihood matrix
# Estimate    SE
# elpd_loo  -1208.0  50.2
# p_loo        30.1   5.7
# looic      2416.1 100.5

loo.annvar  # min looic is 2415.37
#                     LOOIC    SE
# mf.maxt           2415.37 98.94
# mf.mean           2419.49 98.46
# mf.prec           2419.49 98.89
# mf.maxt - mf.mean   -4.12  7.52
# mf.maxt - mf.prec   -4.12  7.13
# mf.mean - mf.prec    0.00  4.18

# Procede with the maximum temperature model.
# This also outperforms the annual variation model.

## Now, do model selection, dropping terms.

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn3TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "loghdCt"),
        set_prior("normal(0, 5)", class = "b", coef = "max.temp"))

# Fit model without headCount 2 (i.e., test for seed bank)
m.b123.h = brm(slingCt ~ burn1 + burn2 + burn3 + log(hdCt) + 
                         max.temp + (1 | site),
               data = recruit.data, 
               family = negbinomial(), prior = pri, 
               cores = 4, seed = 300538)

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "loghdCt"),
        set_prior("normal(0, 5)", class = "b", coef = "max.temp"))

# Fit model without head count
m.b123 = brm(slingCt ~ burn1 + burn2 + log(hdCt) + max.temp + (1 | site),
             data = recruit.data, 
             family = negbinomial(), prior = pri, 
             cores = 4, seed = 173743)

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "max.temp"))

# Fit model without burn 3
m.b12 = brm(slingCt ~ burn1 + burn2 + max.temp + (1 | site),
             data = recruit.data, 
             family = negbinomial(), prior = pri, 
             cores = 4, seed = 230951)

# Set Bayesian priors 
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "max.temp"))

# Fit model without burn2
m.b1 = brm(slingCt ~ burn1 + max.temp + (1 | site),
           data = recruit.data, 
           family = negbinomial(), prior = pri, 
           cores = 4, seed = 22021826)

# Compare all of these models
loof = loo(m.b123.h, m.b123, m.b12, m.b1, reloo = TRUE)
loof

#                     LOOIC    SE
# m.b123.h          2413.13 98.84
# m.b123            2412.03 98.93
# m.b12             2411.29 98.30
# m.b1              2418.64 98.73
# m.b123.h - m.b123    1.11  1.72
# m.b123.h - m.b12     1.84  4.25
# m.b123.h - m.b1     -5.51  6.70
# m.b123 - m.b12       0.74  3.97
# m.b123 - m.b1       -6.61  6.48
# m.b12 - m.b1        -7.35  5.33

# Yep - best model features burn1, burn2, max.temp

## Does radius influence model quality?

# Set Bayesian priors
pri = c(set_prior("normal(0, 5)", class = "b", coef = "burn1TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "burn2TRUE"),
        set_prior("normal(0, 5)", class = "b", coef = "max.temp"),
        set_prior("normal(0, 5)", class = "b", coef = "logradius"))

# Fit model with radius term (log because area is a function of radius squared)
m.b12.r = brm(slingCt ~ burn1 + burn2 + max.temp + log(radius) + (1 | site),
              data = recruit.data, 
              family = negbinomial(), prior = pri, 
              cores = 4, seed = 735533)

loor = loo(m.b12.r, reloo = TRUE, seed = 164636)
loor 
# This is comparable to a model without radius

# Computed from 4000 by 1186 log-likelihood matrix
# Estimate   SE
# elpd_loo  -1205.9 49.2
# p_loo      21.6  4.1
# looic      2411.8 98.4

# Do some posterior predictive checks.
ppp = posterior_predict(m.b12, nsamples = 100, seed = 20625)

# Check for overdispersion
bayesplot::ppc_stat(recruit.data$slingCt, ppp, stat = function(x) mean(x == 0))
# Check for upper tails.
bayesplot::ppc_stat(recruit.data$slingCt, ppp, stat = max)

# Occasionally, these draws produce estimates that are way large
# Considering, though, that we had an observation of 47 seedlings
# at one Nessman focal plant (!) I don't think this is a huge problem

# Now look at mean of posterior draws
bayesplot::ppc_stat(recruit.data$slingCt, ppp, stat = mean)

# Pick 
ranef(m.b12)$site 
# lce is the population closest to the mean population


# Each site-year combination, merged with climate data, fed into model to get posterior predictions
# used in parameterizing matrices

backbone = expand.grid(site.orig = c("eth", "lce", "lcw", "nrrx", "nwlf", "nnwlf", "rrxdc", "sgc", "yoh"),
                 year = 2000:2014) %>%
  # Merge site-year combinations with climate estimates
  merge(read.csv('annual_climate_2000_2017.csv') %>% select('season', 'max.temp'),
        by.x = 'year', by.y = 'season', all.x = TRUE) %>%
  # Add burn status for each site
  # Also, for populations _not_ in this dataset, change site to 'lce'
  mutate(site.orig = as.character(site.orig),
         burn = site.orig %in% c('lcw', 'yoh', 'eth') & year %in% 2003 |
           site.orig %in% 'nnwlf' & year %in% c(2000, 2002, 2010) |
           site.orig %in% 'rrxdc' & year %in% 2005,
         burn1 = site.orig %in% c('lcw', 'yoh', 'eth') & year %in% 2004 |
           site.orig %in% 'nnwlf' & year %in% c(2001, 2003, 2011) |
           site.orig %in% 'rrxdc' & year %in% 2006,
         burn2 = site.orig %in% c('lcw', 'yoh', 'eth') & year %in% 2005 |
           site.orig %in% 'nnwlf' & year %in% c(2004, 2012) |
           site.orig %in% 'rrxdc' & year %in% 2007,
         site  = ifelse(site.orig %in% c('nnwlf', 'nwlf', 'sgc', 'eth', 'lcw'),
                        site.orig, 'lce'),
         radius = 41)

# 4000 posterior samples for each of the aboe site-year combinations
prds = posterior_predict(m.b12, newdata = backbone, seed = 1110011)

prds

# Bootstrapping fecundity posterior estimates to generate 1000 means
# then, reformatting them and removing extraneous columns
recruitment.ests = cbind(backbone,
                    replicate(apply(prds, 2, function(x) mean(sample(x, replace = TRUE))),
                    n = 100)) %>%
  # Convert to long format
  gather(key = trial, value = estimate, as.character(1:100)) %>%
  # Convert burn estimates to zeros
  mutate(estimate = ifelse(burn, 0, estimate)) %>%
  select(-c(site, burn, burn1, burn2, radius, max.temp)) %>%
  rename(site = site.orig)

dim(recruitment.ests)

head(recruitment.ests)

hist(recruitment.ests$estimate)

# Export
write.csv(recruitment.ests, row.names = FALSE, file = 'recruitment_estimates_bayesian.csv')

sessionInfo()

# Session Info for the date these models were run (February, 2019).

# R version 3.5.0 (2018-04-23)
# Platform: x86_64-apple-darwin15.6.0 (64-bit)
# Running under: macOS High Sierra 10.13.6
# 
# Matrix products: default
# BLAS: /System/Library/Frameworks/Accelerate.framework/Versions/A/Frameworks/vecLib.framework/Versions/A/libBLAS.dylib
# LAPACK: /Library/Frameworks/R.framework/Versions/3.5/Resources/lib/libRlapack.dylib
# 
# locale:
# [1] en_US.UTF-8/en_US.UTF-8/en_US.UTF-8/C/en_US.UTF-8/en_US.UTF-8
# 
# attached base packages:
# [1] stats     graphics  grDevices utils     datasets  methods   base     
# 
# other attached packages:
# [1] brms_2.7.0     Rcpp_1.0.0     tidyr_0.8.1    bindrcpp_0.2.2 dplyr_0.7.8    ggplot2_3.1.0 
# 
# loaded via a namespace (and not attached):
# [1] Brobdingnag_1.2-6    gtools_3.8.1         StanHeaders_2.17.2   threejs_0.3.1       
# [5] shiny_1.1.0          assertthat_0.2.0     stats4_3.5.0         yaml_2.1.19         
# [9] backports_1.1.2      pillar_1.3.0.9001    lattice_0.20-35      glue_1.3.0          
# [13] digest_0.6.18        RColorBrewer_1.1-2   promises_1.0.1       colorspace_1.3-2    
# [17] htmltools_0.3.6      httpuv_1.4.5         Matrix_1.2-14        plyr_1.8.4          
# [21] dygraphs_1.1.1.6     pkgconfig_2.0.2      rstan_2.17.3         purrr_0.2.5         
# [25] xtable_1.8-3         mvtnorm_1.0-8        scales_1.0.0         later_0.7.5         
# [29] tibble_1.4.99.9005   bayesplot_1.6.0      DT_0.4               withr_2.1.2         
# [33] shinyjs_1.0          lazyeval_0.2.1       magrittr_1.5         crayon_1.3.4        
# [37] mime_0.5             nlme_3.1-137         xts_0.11-1           colourpicker_1.0    
# [41] rsconnect_0.8.8      tools_3.5.0          loo_2.0.0            matrixStats_0.54.0  
# [45] stringr_1.3.1        munsell_0.5.0        compiler_3.5.0       rlang_0.3.1         
# [49] grid_3.5.0           ggridges_0.5.0       rstudioapi_0.7       htmlwidgets_1.2     
# [53] crosstalk_1.0.0      igraph_1.2.1         miniUI_0.1.1.1       base64enc_0.1-3     
# [57] labeling_0.3         codetools_0.2-15     gtable_0.2.0         inline_0.3.15       
# [61] abind_1.4-5          markdown_0.8         reshape2_1.4.3       R6_2.3.0            
# [65] gridExtra_2.3        rstantools_1.5.1     zoo_1.8-4            bridgesampling_0.6-0
# [69] bindr_0.1.1          shinystan_2.5.0      shinythemes_1.1.1    stringi_1.2.2       
# [73] parallel_3.5.0       tidyselect_0.2.5     coda_0.19-1  

# Final model output.

summary(m.b12)

# Family: negbinomial 
# Links: mu = log; shape = identity 
# Formula: slingCt ~ burn1 + burn2 + max.temp + (1 | site) 
# Data: recruit.data (Number of observations: 1186) 
# Samples: 4 chains, each with iter = 2000; warmup = 1000; thin = 1;
# total post-warmup samples = 4000
# 
# Group-Level Effects: 
# ~site (Number of levels: 17) 
#               Estimate Est.Error l-95% CI u-95% CI Eff.Sample Rhat
# sd(Intercept)     1.01      0.30     0.56     1.73        896 1.00
# 
# Population-Level Effects: 
#           Estimate Est.Error l-95% CI u-95% CI Eff.Sample Rhat
# Intercept    -0.99      0.31    -1.69    -0.44        974 1.00
# burn1TRUE     1.71      0.50     0.77     2.73       2085 1.00
# burn2TRUE     1.51      0.66     0.29     2.91       2795 1.00
# max.temp      0.07      0.03     0.02     0.12       4000 1.00
# 
# Family Specific Parameters: 
#       Estimate Est.Error l-95% CI u-95% CI Eff.Sample Rhat
# shape     0.22      0.02     0.18     0.26       4000 1.00
# 
# Samples were drawn using sampling(NUTS). For each parameter, Eff.Sample 
# is a crude measure of effective sample size, and Rhat is the potential 
# scale reduction factor on split chains (at convergence, Rhat = 1).
