# This script runs multi-stage mark recapture models used to estimate adult
# vital rates.
# These vital rates were used for quantifying fire effects on adult demography
# and population growth in Nordstrom et al., 2021.
# Questions: scottwatsonnordstrom [at] gmail [dot] com

library(tidyr)
library(dplyr)
library(rstan)

rm(list = ls())

# Use these lines to parallelize model fitting.
rstan_options(auto_write = TRUE)
options(mc.cores = 4)

##### Load in demography data.
# IMPORTANT NOTE:
# This includes demography records from NINE remnant populations.
# Originally, we analyzed nine separate populations. Four of these populations
# experienced no burns during our observation period and were intended as
# controls for an earlier version of our analysis, a deterministic LTRE.
# However, recommendations from reviewers and a change in our analysis plans
# convinced us (SN, AD, SW) that the four control populations were superfluous.
# As such, their analysis was NOT included in the final analysis included in the
# manuscript. However, we kept these in this file (and others) in case they may
# be of future use to others.

# Here is a CSV with adult demography data.
adult.demo = read.csv('https://www.dropbox.com/s/wxzerd9yytmoaxn/adult_demography_for_MSMR.csv?raw=1')

# FIELDS:
# each row corresponds to a demographic record
# aka - a unique identifier for each plant
# site - population of focal plant
# year
# ld - was plant alive during this observation (boolean - 0 or 1)
# fl - did plant flower during observation (boolean - 0 or 1)
# hdCt - how many heads did the plant produce (integer)

# Number of records
nrow(adult.demo)
# 8494 records (in NINE sites)

# Records per site
table(adult.demo$site)
# eth   lce   lcw nnwlf  nrrx  nwlf rrxdc   sgc   yoh 
# 213  2957  1300   334  1526  1001   292   220   651 
# Note: final analysis included only ETH, LCW, NNWLF, RRXDC, YOH
# upon reviewer suggestion we abbreviated NNWLF to NNW and
# RRXDC to RRX

##### Reshape data to conform to STAN file (which reads in wide-form data)

demo.wide = adult.demo %>%
  # Create a 'status' field for each record.
  # 1 coresponds to a basal plant
  # 2 is a flowering plant
  # 3 is an observed dead plant
  mutate(status = ifelse(!ld, 3, ifelse(!fl, 1, 2))) %>%
  # Remove extraneous comments
  select(-c(ld, fl, hdCt)) %>%
  # Convert data into wide form
  spread(key = year, value = status) %>%
  # For each missing observation, replace NAs with 0s
  # 0s correspond to missing observations (unknown status)
  # in the model.
  mutate_at(as.character(1998:2017), replace_na, 0)

# Set a fixed seed for each population.
seeds = c(13569707, 285175702, 824516255, 
          293227429, 745751899, 86591213, 
          542821364, 305402849, 758701395)

###### Run through Multi-stage Mark Recapture model

# Pick seed index
# ENSURE THAT THIS IS RUN if running model multiple times
seed.ind = 1

# Iterate over each site
for(s in unique(demo.wide$site)) {
  
  # Subset demo wide for site
  dow = demo.wide[demo.wide$site %in% s,]
  
  # Initialize Stan file input
  # T_years is number of observed transitions
  # N_plant is how many plants are in the population
  # y is actual data
  assign(paste0('stan_', s), list(T_years = 20,
                                  N_plant = nrow(dow),
                                  y = dow %>% select(-c(aka, site))))
  
  print(s)
  
  # Fit the stan file, assign it to an object with '_mod' suffix
  # e.g., rrxdc_mod (= site RRX in manuscript)
  system.time(assign(paste0(s, '_mod'), stan(file = '02_vital_rate_models/multistage_mark_recapture.stan', 
                                             data = get(paste0('stan_', s)),
                                             seed = seeds[seed.ind])))
  
  # Get the next seed.
  seed.ind = seed.ind + 1
  
}

# Save all models
save.image(file = '03_vital_rate_posteriors/MSMR_raw_output.RData')

# Note: re-running this file before upload produced the file 
# '02_vital_rate_models/multistage_mark_recapture.rds'
# which was not used in analysis at any point

# To extract model output and subsample posterior estimates, see script
# `02_vital_rate_models/extract_adult_rates_bayesian.R`
