#####
# Script for assembling matrices used in matrix analysis in Nordstrom et al. 2021
# It reads in posterior estimates and outputs 200 matrices per site-year combination.
# 
# Note: matrices here include four additional populations included in preliminary analysis.
# These populations did not experience burns during our observation window and
# were meant to serve as a control.
# We removed these populations from analysis based on the advice of reviewers.
# However, we leave these in these scripts for completeness and in case they
# may be of use to future users.
# 
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####


##### Setup

library(ggplot2)
library(dplyr)
library(tidyr)

# Clear namespace
rm(list = ls())

# Read in wrapper functions for constructing matrices
source('00_global/functionsForLTRE.R')

##### First step - do high juvenile survival matrices

# Read in posterior estimates
# (note: reading in orignal estimates from analysis - 
# changes to brms and R may produce slightly different estimates)
juv.surv = read.csv('03_vital_rate_posteriors/juv_surv_bayesian_hijuv_original.csv') # note - high juv surv. estimates only
juv.flow = read.csv('03_vital_rate_posteriors/juv_flower_bayesian_original.csv') 
seed.est = read.csv('03_vital_rate_posteriors/recruit_estimates_bayesian_original.csv')
adult.dm = read.csv('03_vital_rate_posteriors/adult_rates_bayesian_original.csv')

# Create an object with all population-year-burn combos 
# This will be the basis of a sequence of merges
backbone = expand.grid(site = c("eth", "lce", "lcw", "nrrx", "nwlf", "nnwlf", "rrxdc", "sgc", "yoh"),
                       year = 2000:2014) %>%
  mutate(burn = site %in% c('yoh', 'eth', 'lcw') & year %in% 2003 |
           site %in% 'nnwlf' & year %in% c(2000, 2002, 2010) |
           site %in% 'rrxdc' & year %in% 2005,
         burn1 = site %in% c('yoh', 'eth', 'lcw') & year %in% 2004 |
           site %in% 'nnwlf' & year %in% c(2001, 2003, 2011) |
           site %in% 'rrxdc' & year %in% 2006,
         burn2 = site %in% c('yoh', 'eth', 'lcw') & year %in% 2005 |
           site %in% 'nnwlf' & year %in% c(2004, 2012) |
           site %in% 'rrxdc' & year %in% 2007)

# Format the juvenile survival info
juv.surv = juv.surv %>%
  # age4 is the predictor used in our analysis 
  # (age classes 0, 1, 2+, labelled in data as 1, 2, 3+)
  spread(key = age4, value = estimate) %>%
  # Rename columns to three matrix classes
  rename(s_s = `1`,
         s_j1 = `2`,
         s_j2 = `3+`) %>%
  # Annual survival for juvenile stages 2 and 3 is same
  mutate(s_j3 = s_j2) %>%
  # Remove unnecessary columns
  select(-c(total.precip, mean.temp, prior, plot, unitID, zone))

# Format the juvenile flowering info so that estimates are paired with population-years
juv.flow = juv.flow %>%
  merge(y = backbone, by = c("burn", "year"), all.y = TRUE) %>%
  rename(f_j3 = estimate)

# Format adult vital rate info
adult.dm = adult.dm %>%
  select(-c(param, vital, stage)) %>%
  spread(key = rate, value = estimate)

# Format seedling recruitment info (jus renaming a column)
seed.est = seed.est %>% rename(r_f = estimate)

# Merge all info together to get vital rate esimates (100 each) for
# each popultion in each year.
all.rates = merge(juv.flow, adult.dm, by = c('site', 'year', 'trial')) %>%
  merge(y = seed.est, by = c('site', 'year', 'trial')) %>%
  merge(y = juv.surv, by = c('year', 'trial', 'burn')) %>%
  # Note: slight misnomer - alpha here is stage of J2 (five years - between ages 2 and 7)
  # alpha in this case does not mean age of first reproduction
  mutate(alpha = 5) %>%
  select(year, site, trial, burn, burn1, burn2, 
         f_b, f_f, f_j3, s_b, s_f, s_s, s_j1, s_j2, s_j3, r_f, alpha)

# Read in wrapper function to convert each row to a matrix
all.matr = all.rates[,-(1:6)] %>%
  apply(1, function(x) matrixFromRates(as.list(x)) %>% matrix(ncol = 1))

# Write CSV files
write.csv(all.rates, row.names = FALSE, file = '05_matrix_output/all_vital_rates_bayesian_hijuv.csv')
write.csv(all.matr, row.names = FALSE, file = '05_matrix_output/all_matrices_bayesian_hijuv.csv')

##### Simulated vital rates for the rate decompostion procedure

sim.backbone = expand.grid(site = c("eth", "lce", "lcw", "nrrx", "nwlf", "nnwlf", "rrxdc", "sgc", "yoh"),
                           burn = c(TRUE, FALSE),
                           burn1 = c(TRUE, FALSE),
                           burn2 = c(TRUE, FALSE),
                           year = 2000:2014) %>%
  filter(burn + burn1 + burn2 < 2)

sim.seed.est = read.csv('sim_fecundity_estimates_bayesian.csv') %>%
  rename(r_f = estimate)

sim.juv.flow = read.csv('juv_flower_bayesian.csv') %>%
  merge(y = expand.grid(burn = c(TRUE, FALSE), burn1 = c(TRUE, FALSE), burn2 = c(TRUE, FALSE)),
        by = "burn", all.x = TRUE) %>%
  filter(burn + burn1 + burn2 < 2) %>%
  arrange(trial, burn, burn1, burn2, year) %>%
  rename(f_j3 = estimate)

sim.juv.surv = read.csv('juv_surv_bayesian_rct.csv') %>%
  select(year, burn, age4, trial, estimate) %>%
  merge(y = expand.grid(burn = c(TRUE, FALSE), burn1 = c(TRUE, FALSE), burn2 = c(TRUE, FALSE)),
        by = "burn", all.x = TRUE) %>%
  filter(burn + burn1 + burn2 < 2) %>%
  rename(age = age4) %>%
  spread(key = age, value = estimate) %>%
  rename(s_s = `1`,
         s_j1 = `2`,
         s_j2 = `3+`) %>%
  mutate(s_j3 = s_j2) %>%
  arrange(trial, year, burn, burn1, burn2)

sim.rates = sim.backbone %>%
  merge(y = sim.juv.flow, by = c("burn", "burn1", "burn2", "year")) %>%
  merge(y = sim.juv.surv, by = c("burn", "burn1", "burn2", "year", "trial")) %>%
  merge(y = sim.seed.est, by = c("burn", "burn1", "burn2", "year", "site", "trial")) %>%
  merge(y = merge(adult.dm, backbone, by = c("site", "year")), 
        by = c("burn", "burn1", "burn2", "site", "year", "trial"), all.x = TRUE) %>%
  arrange(site, year, trial, burn2, burn1, burn)

write.csv(sim.rates, row.names = FALSE, file = 'simulated_vital_rates_bayesian_rct.csv')

##### Construct matrices with the low juvenile survival estimtes
# Note: don't clear namsepace. Namespace from above is needed for this.

# Read in estimates and reformt for merge
juv.surv = read.csv('03_vital_rate_posteriors/juv_surv_bayesian_lojuv_original.csv')  %>%
  spread(key = age3, value = estimate) %>%
  rename(s_s  = `1`,
         s_j1 = `2+`) %>%
  # According to low survival model, all ages older than 1 year old are same
  mutate(s_j2 = s_j1,
         s_j3 = s_j1)

# Format matrices
all.rates = merge(juv.flow, adult.dm, by = c('site', 'year', 'trial')) %>%
  merge(y = seed.est, by = c('site', 'year', 'trial')) %>%
  merge(y = juv.surv, by = c('site', 'year', 'trial')) %>%
  # See note above about alpha (corresponds here to stage duration
  # and not time until first reproduction)
  mutate(alpha = 5)

# Construct matrices
all.matr = all.rates[,-c(1:4, 6:7)] %>%
  apply(1, function(x) matrixFromRates(as.list(x)) %>% matrix(ncol = 1))

write.csv(all.rates, row.names = FALSE, file = '05_matrix_output/all_vital_rates_bayesian_lojuv.csv')
write.csv(all.matr, row.names = FALSE, file = '05_matrix_output/all_matrices_bayesian_lojuv.csv')

##### Simulations for rate decomposition procedure with low juvenile survival dataset

sim.backbone = expand.grid(site = c("eth", "lce", "lcw", "nrrx", "nwlf", "nnwlf", "rrxdc", "sgc", "yoh"),
                           burn = c(TRUE, FALSE),
                           burn1 = c(TRUE, FALSE),
                           burn2 = c(TRUE, FALSE),
                           year = 2000:2014) %>%
  filter(burn + burn1 + burn2 < 2)

sim.seed.est = read.csv('sim_fecundity_estimates_bayesian.csv') %>%
  rename(r_f = estimate)

sim.juv.flow = read.csv('juv_flower_bayesian.csv') %>%
  merge(y = expand.grid(burn = c(TRUE, FALSE), burn1 = c(TRUE, FALSE), burn2 = c(TRUE, FALSE)),
        by = "burn", all.x = TRUE) %>%
  filter(burn + burn1 + burn2 < 2) %>%
  arrange(trial, burn, burn1, burn2) %>%
  rename(f_j2 = estimate)

sim.rates = sim.backbone %>%
  merge(y = sim.juv.flow, by = c("burn", "burn1", "burn2", "year")) %>%
  merge(y = sim.seed.est, by = c("burn", "burn1", "burn2", "year", "site", "trial")) %>%
  merge(y = merge(adult.dm, backbone, by = c("site", "year")), 
        by = c("burn", "burn1", "burn2", "site", "year", "trial"), all.x = TRUE) %>%
  arrange(site, year, trial, burn2, burn1, burn)

write.csv(sim.rates, row.names = FALSE, file = 'simulated_vital_rates_bayesian_ref.csv')
