#####
# File for permutation tests, testing for differences in adult flowering and
# survival with time since fire for Nordstrom et al. 2021.
# This generates a table (original results printed as comment) appearing in supporting information.
# Due to deprecations in tidyr/dplyr I have made some changes since original anaylysis in 2019.
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####

# Load useful packages
library(ggplot2)
library(dplyr)
library(tidyr)

# Clear namespace
rm(list = ls())

# Define burn years and years one year after burns
burn.yrs = c('nnwlf2000', 'nnwlf2002', 'nnwlf2010', 'yoh2003', 'eth2003', 'rrxdc2005', 'lcw2003')
burn1yrs = c('nnwlf2001', 'nnwlf2003', 'nnwlf2011', 'yoh2004', 'eth2004', 'rrxdc2006', 'lcw2004')

# Read in vital rate data (subsampld posterior estimates for each site-year)
adult.rates = read.csv('03_vital_rate_posteriors/adult_rates_bayesian_original.csv') %>%
  # Get only sites of interest
  filter(site %in% c('lcw', 'yoh', 'eth', 'nnwlf', 'rrxdc')) %>%
  # Remove unnecessary columns
  select(-c(param, vital, stage)) %>%
  # Convert to wide form
  spread(key = rate, value = estimate)
# Fields:
#   trial - number of posterior estimate (1-100)
#   year - year of observation
#   site - population
#   f_b, f_f - respective estimate of flowering for basal and flowering plants (resp.)
#   s_b, s_f - respective estimate of survival  for basal and flowering plants (resp.)

# Functions for getting quantiles defining a 95% interval
q.025 = function(x) quantile(x, 0.025)
q.975 = function(x) quantile(x, 0.975)

# Chain to get median and quantile estimates for each burn status

adult.medians = adult.rates %>%
  gather(key = rate, value = est, -c(trial, year, site)) %>%
  # year of burn gets burn0 = TRUE, year after burn gets burn1 = TRUE
  mutate(burn0 = paste0(site, year) %in% burn.yrs, 
         burn1 = paste0(site, year) %in% burn1yrs) %>%
  # Summarise - get median rate and quantiles defining 95% CI
  group_by(site, year, rate) %>%
  summarise(med = median(est),
            q05 = quantile(est, 0.025),
            q95 = quantile(est, 0.975),
            # get burn status of that population in that year
            burn = ifelse(any(burn0), 'Burn',
                          ifelse(any(burn1), 'Burn1', 
                                 'No burn'))) %>%
  # These are useful for the plot:
  #   rate is a label for the plot, texty is a y cord for location of label on plot
  merge(data.frame(rate = c('f_b', 'f_f', 's_b', 's_f'),
                   texty = c(0.75, 0.75, 0.55, 0.55)),
        by = 'rate')

# Sample median associated with each time since fire category.
adult.medians %>%
  select(-c(q05, q95, texty)) %>%
  mutate(burn = paste0(site, year) %in% burn.yrs,
         burn1 = paste0(site, year-1) %in% burn.yrs,
         burn2 = ifelse(burn, FALSE, paste0(site, year-2) %in% burn.yrs),
         noburn = !(burn | burn1 | burn2)) %>%
  gather(key = burn.status, value = status.is, -c(rate, site, year, med)) %>%
  filter(status.is) %>%
  select(-status.is) %>%
  group_by(rate, burn.status) %>%
  summarise(med = median(med)) %>%
  spread(key = burn.status, value = med)

#####
# Analysis comparing all four time-since-fire categories

# Long-form for permutations
adult.rates.for.permute = adult.rates %>%
  gather(key = rate, value = est, -c(trial, year, site)) %>%
  mutate(burn = paste0(site, year) %in% burn.yrs,
         burn1 = paste0(site, year-1) %in% burn.yrs,
         burn2 = ifelse(burn, FALSE, paste0(site, year-2) %in% burn.yrs),
         no.burn = !(burn | burn1 | burn2)) %>%
  gather(key = status, value = status.is, c(burn, burn1, burn2, no.burn)) %>%
  filter(status.is) %>% select(-status.is)

# Observed medians
permute.01 = adult.rates.for.permute %>% 
  group_by(rate, status) %>%
  summarise(m = median(est))

### Below, getting a null distribution of medians

# Initialize an object for storing results
permute.results = expand.grid(rate = unique(permute.01$rate),
                              status1 = unique(permute.01$status), 
                              status2 = unique(permute.01$status))

set.seed(3905584)

for (trial in 1:1000) {
  
  # Sample medians by re-assigning times since fire (status column)
  # to site year combinations
  samp = adult.rates.for.permute %>%
    distinct(site, year, status) %>% group_by(site) %>%
    mutate(status = sample(status, replace = FALSE))
  
  # Merge full list of re-assigned statuses to the full array of vital rate estimates
  # and calculate medians
  permute.00 = adult.rates %>%
    group_by(trial) %>%
    merge(y = samp, by = c("site", "year")) %>%
    gather(key = rate, value = est, contains('_')) %>%
    # Get median estimate for each status
    group_by(rate, status) %>%
    summarise(m = median(est)) %>%
    # Here - get pairwise differences
    merge(x = ., y = ., by = 'rate') %>%
    rename(status1 = status.x, status2 = status.y) %>%
    # d = difference between groups
    mutate(d = m.x - m.y)
  
  # Add the above to the object 
  permute.results = merge(x = permute.results, 
                          y = permute.00 %>% select(-c(m.x, m.y)), 
                          by = c("rate", "status1", "status2")) %>%
    # NOTE: original script had the below:
    # rename_at(vars(d), funs( paste0('null', trial) ) )
    # however, dplyr syntax changed and now requires an implementation with list()
    rename_at(vars(d), list(~ paste0('null', trial)))
  
  print(trial)
  
}

# Merge results data frame with observed differences to compare observed with null
# first four lines are getting observed pairwise differences
permute.results = merge(permute.01, permute.01, by = 'rate') %>%
  rename(status1 = status.x, status2 = status.y) %>%
  mutate(obs.diff = m.x - m.y) %>%
  select(-c(m.x, m.y)) %>%
  # Merging with null distribution of results
  merge(x = permute.results, y = ., by = c("rate", "status1", "status2")) %>%
  # We don't need cases comparing the same status (medians will be identical)
  filter(status1 != status2) %>%
  # Clunky, but, getting the proportion of null differences greater than observed
  # nullmax and nullmin are the greatest and smallest null differences
  mutate(p = apply(., 1, function(x) mean( as.numeric(x[paste0('null', 1:1000)]) > as.numeric(x['obs.diff']))),
         nullmax = apply(., 1, function(x) max( as.numeric(x[paste0('null', 1:1000)]))),
         nullmin = apply(., 1, function(x) min( as.numeric(x[paste0('null', 1:1000)]))))

# Print four digits (for readibility)
permute.results[,c("rate", "status1", "status2", "obs.diff", "nullmax", "nullmin", "p")] %>%
  arrange(desc(obs.diff / nullmax)) %>% mutate_if(is.numeric, round, 4)

permute.results[,c("rate", "status1", "status2", "obs.diff", "nullmax", "nullmin", "p")] %>%
  arrange(p) %>% mutate_if(is.numeric, function(x) round(x, 3)) %>%
  filter(obs.diff > 0)
# (Can filter like this b/c, for any negative, there is a positive included b/c
# each pair is included twice with signs reversed)

# Output from original run of this (Spring 2019): 

# rate status1 status2 obs.diff nullmax nullmin     p
# 1   f_b    burn   burn2    0.226   0.207  -0.281 0.000
# 2   f_b    burn no.burn    0.183   0.197  -0.098 0.001
# 3   f_f    burn   burn2    0.223   0.271  -0.240 0.002
# 4   f_f    burn   burn1    0.217   0.269  -0.237 0.003
# 5   f_f    burn no.burn    0.168   0.209  -0.103 0.004
# 6   s_f   burn1 no.burn    0.030   0.042  -0.059 0.008
# 7   s_f   burn1   burn2    0.049   0.079  -0.070 0.013
# 8   s_f    burn   burn2    0.044   0.082  -0.070 0.018
# 9   s_f    burn no.burn    0.025   0.041  -0.050 0.025
# 10  f_b    burn   burn1    0.134   0.243  -0.212 0.026
# 11  f_f no.burn   burn1    0.049   0.102  -0.189 0.057
# 12  f_b   burn1   burn2    0.092   0.236  -0.258 0.077
# 13  f_f no.burn   burn2    0.054   0.113  -0.209 0.087
# 14  s_f no.burn   burn2    0.019   0.066  -0.042 0.154
# 15  f_b no.burn   burn2    0.044   0.088  -0.258 0.166
# 16  f_b   burn1 no.burn    0.048   0.214  -0.105 0.191
# 17  s_b   burn2   burn1    0.020   0.067  -0.061 0.195
# 18  s_b no.burn   burn1    0.018   0.058  -0.027 0.203
# 19  s_b    burn   burn1    0.013   0.076  -0.069 0.262
# 20  s_b   burn2 no.burn    0.002   0.026  -0.059 0.347
# 21  s_b   burn2    burn    0.007   0.077  -0.065 0.389
# 22  s_f   burn1    burn    0.005   0.060  -0.065 0.398
# 23  s_b no.burn    burn    0.005   0.061  -0.027 0.480
# 24  f_f   burn1   burn2    0.005   0.218  -0.202 0.533
