#####
# Script for calculating stochastic growth rates at varying burn frequencies.
# This dataset is for ONLY the LOW juvenile survival dataset.
# Analysis for Nordstrom et al. 2021
#
# This script was originally run on a Ubuntu server in Spring 2019.
# 
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####

library(dplyr)

# Change namespace
rm(list = ls())

# Read in wrapper functions
source('00_global/functionsForLTRE.R')

# Read in data
all.rates = read.csv('05_matrix_output/all_vital_rates_lojuv_original.csv', 
                     stringsAsFactors = FALSE)
all.matr = read.csv('05_matrix_output/all_matrices_bayesian_lojuv_original.csv', 
                    stringsAsFactors = FALSE)

# Define burn years
burnyrs = all.rates %>% filter(burn) %>% with(paste0(site, year))

# For sites, also define years as one year or two years prior to burn
all.rates = all.rates %>%
  group_by(site) %>%
  mutate(burn1 = paste0(site, year-1) %in% burnyrs,
         burn2 = paste0(site, year-2) %in% burnyrs & !burn)

# Sites to iterate over
# NOTE: these are the only sites experiencing burns
sites = c('eth', 'lcw', 'nnwlf', 'rrxdc', 'yoh')

# res object will will store results
# NOTE: burn frequencies were run between 0% and 90% although only 0-50% were used in analysis
res = expand.grid(site = sites, freq = (0:9) / 10) %>% mutate(mean = NA, sd = NA)
head(res)

# initialize initial condition vectors for each site
inits = rep(1e300, 6) %>% matrix()

##### Run the procedure 
# Looping over each of site and burn frequency, get 500 estimates of log lambda
# take mean and sd of each

set.seed(101101101)

for (i in 1:nrow(res)) {
  
  # eigeth will store eigenvalues for each of 500 trials
  eigeth = vector('numeric', 500)
  
  # site and probability
  s = res$site[i]
  p = res$freq[i]
  
  for (j in 1:500) {
    
    # Generate a fire sequence of 1500 years (sim.seq is a wrapper from wrapper script)
    fire.seq = sim.seq(p, 1500)
    
    # sample matrix indices in accordance with burn sequence
    # (i.e., for a burn year, sample from matrices of non-burn years, for burn year, sample from burn years...)
    inds = ifelse(!fire.seq, sample(which(with(all.rates, site %in% s & !(burn | burn1 | burn2))), 
                                    size = sum(!fire.seq), replace = TRUE),
                  ifelse(fire.seq == 1, sample(which(with(all.rates, site %in% s & burn)), 
                                               size = sum(fire.seq == 1), replace = TRUE),
                         ifelse(fire.seq == 2, sample(which(with(all.rates, site %in% s & burn1)), 
                                                      size = sum(fire.seq == 2), replace = TRUE),
                                sample(which(with(all.rates, site %in% s & burn2)), 
                                       size = sum(fire.seq == 3), replace = TRUE))))
    
    # make 25 x 5000 array of matrices (each column has the matrix entries of one year in the burn seq.)
    m = all.matr[, inds]
    
    # p0 is the population at time 0 (i.e., initial condition)
    p0 = inits
    # p1 is the population after a burn-in period (after 1000 years)
    p1 = matr.prod(m[,1:500]) %*% p0
    # p2 is the population 4000 years after the burn-in period (after 5000 years)
    p2 = matr.prod(m[,501:1500]) %*% p1
    
    # the log-growth rate is the mean difference in log population size (log-ratio) over the 4000 years
    eigeth[j] = (log(sum(p2)) - log(sum(p1))) / 1000
    
  }
  
  res[i, c("mean", "sd")] = c(mean(eigeth), sd(eigeth))
  print(i)
  
}

write.csv(res, row.names = FALSE, file = '07_analysis_outputs/growth_by_burn_freq_bayesian_lojuv.csv')

