#####
# Script performing the rate decomposition analysis with LOW juvenile survival data.
# Analysis for Figure 5 of Nordstrom et al. 2021
#
# Analysis originally performed in Spring 2019 on an ubuntu server
# 
# Questions: scottwatsonnordstrom [at] gmail.com
#####

##### Setup

library(dplyr)

# Clear namespace
rm(list = ls())

# Add wrapper functions to namespace
source('00_global/functionsForLTRE.R')

# Load data

# Vital rates used to fit matrices
all.rates = read.csv('05_matrix_output/all_vital_rates_lojuv_original.csv', 
                     stringsAsFactors = FALSE)
# Matrices themselves
all.matr = read.csv('05_matrix_output/all_matrices_bayesian_lojuv_original.csv', 
                    stringsAsFactors = FALSE)

# Simulated vital rates
#   These are needed to avoid confounding burn and year in the rate decomposition.
#   They were generated by getting model predictions for each site + year
#   evaluated with burn = TRUE (or some other post burn year value)
sim.rates = read.csv('05_matrix_output/simulated_vital_rates_lojuv_original.csv', 
                     stringsAsFactors = FALSE)

# Designate burn years
burnyrs = all.rates %>% filter(burn) %>% with(paste0(site, year))

# Label vital rates according to burn status
all.rates = all.rates %>%
  group_by(site) %>%
  mutate(burn1 = paste0(site, year-1) %in% burnyrs,
         burn2 = paste0(site, year-2) %in% burnyrs & !burn) %>%
  # Rename columns (accidental discrepancy in naming)
  rename(s_s  = s_s1,
         s_j1 = s_s2,
         s_j2 = s_j1,
         s_j3 = s_j2,
         f_j3 = f_j2)

# Sites to perform procedure over
sites = c('eth', 'lcw', 'nnwlf', 'rrxdc', 'yoh')

# Initialize object to store results in
# store results for each combination of population, rate, and burn frequency
# Note: the procedure runs for 0.2 but only burn frequencies of 0.1 and 0.5
# are used in analysis.
ras = expand.grid(site = sites, prob = c(0.10, 0.20, 0.50), 
                  rate = grep('\\_', names(sim.rates), value = TRUE)) %>%
  mutate(median = NA, mad = NA)

# Initialize populations (this is N_0)
inits = rep(1e200, 6) %>% matrix()

### Run rate decomposition procedure

set.seed(352669)

for (i in 1:nrow(ras)) {
  
  s = ras$site[i] # population
  p = ras$prob[i] # burn frequency
  r = ras$rate[i] # vital rate
  site.rates = sim.rates[sim.rates$site %in% s,] # object with vital rates just at given population
  
  # Store population growth rates for 500 trials
  eigeth = vector('numeric', 500)
  
  for (j in 1:500) {
    
    # burn sequence of length 1500 years
    seqq = sim.seq(p, 1500)
    
    ### Burn sequence in absence of fire
    
    # Indices of matrices to include in sequence, sampling all matrices only from the non-burn years
    inns = sample(which(with(all.rates, site %in% s & !(burn | burn1 | burn2))), length(seqq), replace = TRUE)
    
    m = all.matr[, inns]
    
    # initial condition (population at time 0)
    p0 = inits
    # population after burn in period (500 years)
    p1 = matr.prod(m[,1:500]) %*% p0
    # population 1000 years after burn in period
    p2 = matr.prod(m[,501:1500]) %*% p1
    
    # Population growth rate over this period
    # (this is c_0 in the paper)
    ll.old = (log(sum(p2)) - log(sum(p1))) / 1000
    
    ### Below: calculate growth rate substituting in the vital rate of interest (r)
    # in respective burn years according to the burn sequence.
    
    # for burn years, sample the rate of interest from those observed in burn periods
    m[, seqq == 1] = apply(all.rates[inns[seqq == 1], -(1:2)], 1, FUN = subs.rate, rate.name = as.character(r),
                           rate.val = site.rates[site.rates$burn, as.character(r)])
    # for one year after burn, sample rate of interest from those observed one year after burns
    m[, seqq == 2] = apply(all.rates[inns[seqq == 2], -(1:2)], 1, FUN = subs.rate, rate.name = as.character(r),
                           rate.val = site.rates[site.rates$burn1, as.character(r)])
    # for two years after burn, sample rate of interest rom those observed two years after
    m[, seqq == 3] = apply(all.rates[inns[seqq == 3], -(1:2)], 1, FUN = subs.rate, rate.name = as.character(r),
                           rate.val = site.rates[site.rates$burn2, as.character(r)])
    
    # initial condition (population at time 0)
    p0 = inits
    # population after burn in period (1000 years)
    p1 = matr.prod(m[,1:500]) %*% p0
    # population 4000 years after burn in period
    p2 = matr.prod(m[,501:1500]) %*% p1
    
    # Population growth rate
    ll.new = (log(sum(p2)) - log(sum(p1))) / 1000
    
    # store mean difference of log population growth rates (log-ratio)
    eigeth[j] = ll.new - ll.old
    
  }
  
  # Store median and MAD of differences in growth rates
  ras[i, c("median", "mad")] = c(median(eigeth), mad(eigeth))
  print(i)
  
}

write.csv(ras, row.names = FALSE, file = '07_analysis_outputs/rate_decomposition_lojuv.csv')

