#####
# Script to recreate Figure 2 for Nordstrom et al. 2021
# Figure 2 plots posterior estimates of four adult rates (sF, sB, fF, fB)
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####

# Load useful packages
library(ggplot2)
library(lemon) # v 0.4.5
library(dplyr)
library(tidyr)

rm(list = ls())

# Define burn years and years one year after burns
burn.yrs = c('nnwlf2000', 'nnwlf2002', 'nnwlf2010', 'yoh2003', 'eth2003', 'rrxdc2005', 'lcw2003')
burn1yrs = c('nnwlf2001', 'nnwlf2003', 'nnwlf2011', 'yoh2004', 'eth2004', 'rrxdc2006', 'lcw2004')

# Read in vital rate data (subsampld posterior estimates for each site-year)
adult.rates = read.csv('03_vital_rate_posteriors/adult_rates_bayesian_original.csv') %>%
  # Get only sites of interest
  filter(site %in% c('lcw', 'yoh', 'eth', 'nnwlf', 'rrxdc')) %>%
  # Remove unnecessary columns
  select(-c(param, vital, stage)) %>%
  # Convert to wide form
  spread(key = rate, value = estimate)
# Fields:
#   trial - number of posterior estimate (1-100)
#   year - year of observation
#   site - population
#   f_b, f_f - respective estimate of flowering for basal and flowering plants (resp.)
#   s_b, s_f - respective estimate of survival  for basal and flowering plants (resp.)

# Functions for getting quantiles defining a 95% interval
q.025 = function(x) quantile(x, 0.025)
q.975 = function(x) quantile(x, 0.975)

# Chain to get median and quantile estimates for each burn status
adm = adult.rates %>%
  gather(key = rate, value = est, -c(trial, year, site)) %>%
  # year of burn gets burn0 = TRUE, year after burn gets burn1 = TRUE
  mutate(burn0 = paste0(site, year) %in% burn.yrs, 
         burn1 = paste0(site, year) %in% burn1yrs) %>%
  # Summarise - get median rate and quantiles defining 95% CI
  group_by(site, year, rate) %>%
  summarise(med = median(est),
            q05 = quantile(est, 0.025),
            q95 = quantile(est, 0.975),
            # get burn status of that population in that year
            burn = ifelse(any(burn0), 'Burn',
                          ifelse(any(burn1), 'Burn1', 
                                 'No burn'))) %>%
  # # These are useful for the plot:
  #   rate is a label for the plot, texty is a y cord for location of label on plot
  merge(data.frame(rate = c('f_b', 'f_f', 's_b', 's_f'),
                   labs = c('a)', 'b)', 'c)', 'd)')),
                   #texty = c(0.75, 0.75, 0.55, 0.55)),
                   #texty = c(0.9, 0.9, 1.08, 1.08)),
        by = 'rate')

# text labels for plot
scrips = c(expression("a) f"["B"]), expression("b) f"["F"]),
           expression("c) s"["B"]), expression("d) s"["F"])) #%>%
  #rep(each = 75)

ggplot(adm) + 
  geom_errorbar(aes(x = year, ymin = q05, ymax = q95, group = site, colour = burn),
                position = position_dodge(0.4), size = 0.2) +
  geom_point(aes(x = year, y = med, group = site, fill = burn, shape = burn),
             position = position_dodge(0.4), size = 3) +
  geom_point(aes(x = year, y = med, group = site, fill = burn, alpha = burn %in% 'No burn', shape = burn),
             position = position_dodge(0.4), size = 3) +
  geom_point(aes(x = year, y = med, group = site, fill = burn, alpha = !(burn %in% 'Burn'), shape = burn),
             position = position_dodge(0.4), size = 3) +
  scale_alpha_manual(values = c(1, 0)) +
  scale_shape_manual(values = c(24, 22, 21),
                     labels = c('Burn year', 'One year after burn',
                                'Non-burn year')) +
  scale_color_manual(values = c('orangered', 'darkorchid3', 'black'),
                     labels = c('Burn year', 'One year after burn',
                                'Non-burn year')) +
  scale_fill_manual( values = c('orangered', 'darkorchid3', 'black'),
                     labels = c('Burn year', 'One year after burn',
                                'Non-burn year')) +
  scale_x_continuous(limits = c(1999.5, 2015),
                     breaks = 2000 + (0:3)*5,
                     labels = 2000 + (0:3)*5) +
  scale_y_continuous(limits = c(0, 1.11),
                     breaks = (0:4)/4,
                     labels = c('0.00', '0.25', '0.50', '0.75', '1.00')) +
  geom_text(aes(x = 2000, y = 1.09), 
            data = adm %>% distinct(rate),
            label = scrips, size = 4) +
  guides(alpha = 'none', 
         shape = guide_legend(label.position = 'left'),
         color = guide_legend(label.position = 'left'),
         fill  = guide_legend(label.position = 'left')) +
  facet_rep_wrap(~ rate, scales = 'free', repeat.tick.labels = 'right') +
  theme_classic() +
  theme(legend.key = element_blank(),
        legend.justification = 'right',
        legend.position = c(1.00, 0.10),
        legend.title = element_blank(),
        legend.text = element_text(size = 11),
        legend.background = element_blank(),
        axis.ticks.length = unit(0.25, 'cm'),
        axis.title = element_text(size = 15),
        axis.text = element_text(size = 11),
        panel.background = element_blank(),
        panel.border = element_blank(),
        panel.grid = element_blank(),
        strip.background = element_blank(),
        strip.text = element_blank()) +
  xlab('Year') + ylab('Rate estimate') +
  ggsave('09_figure_out/fig_2.png', width = 8, height = 5, units = 'in')


