#####
# Script for re-creating Figure 3 (juvenile survival in burn and non-burn years)
# in Nordstrom et al. 2021
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####

# Load packages
library(ggplot2)
library(lemon) # v 0.4.5
library(dplyr)
library(tidyr)

rm(list = ls())

# Read in high juvenile survival posterior estimates
juvsurv.ests = read.csv('03_vital_rate_posteriors/juv_surv_bayesian_hijuv_original.csv') %>%
  merge(y = expand.grid(age = c('1', '2', '3+'), burn = c(FALSE, TRUE)) %>%
          mutate(lab = paste0(letters[1:6], ')')),
        by.x = c('age4', 'burn'), by.y = c('age', 'burn')) %>%
  mutate(age = ifelse(age4 %in% '1', 'Seedling', ifelse(age4 %in% '2', 'Age 1', 'Ages 2+')) %>%
           factor(levels = c("Seedling", "Age 1", "Ages 2+")), # reorder
         burn = ifelse(burn, 'Burn', 'No burn') %>% factor(levels = c('No burn', 'Burn'))) %>%
  group_by(age, burn) %>%
  # Median estimate for each grouping of age and burn status (burn/noburn)
  mutate(m = median(estimate))

juvsurv.ests %>% 
  # Begin plot
  ggplot() +
  geom_histogram(aes(x = estimate),
                 position = 'identity', 
                 binwidth = 0.025, 
                 colour = 'black', 
                 fill = 'black') +
  geom_segment(aes(x = m, xend = m, 
                   y = 0, yend = 1100), 
               colour = 'red') +
  geom_text(data = juvsurv.ests %>% distinct(lab, .keep_all = TRUE),
            aes(x = 0.46, y = 1000,
                label = paste0(lab, '\n', age, '\n', burn)),
            hjust = 0, size = 3
  ) +
  theme_classic() +
  theme(panel.background = element_blank(),
        strip.background = element_blank(),
        strip.text = element_blank(),
        axis.ticks.length = unit(0.25, 'cm'),
        axis.text  = element_text(size = 11),
        axis.title = element_text(size = 15)) +
  scale_x_continuous(limits = c(0.40, 1.03),
                     breaks = (3:5)/5,
                     labels = c('0.6', '0.8', '1.0')) +
  scale_y_continuous(limits = c(0, 1100)) +
  facet_rep_wrap(burn ~ age, nrow = 2, scales = 'free', 
                 repeat.tick.labels = c('right')) +
  xlab('Survival estimate') +
  ylab('Number of posterior estimates') +
  ggsave('09_figure_out/fig_3.png', width = 8, height = 5)
