#####
# Script for recreating Figure 4 in Nordstrom et al. 2021
# This shows estimated population growth rates for each population
# at annual burn probabilities between 0% and 50%
# Questions: scottwatsonnordstrom [at] gmail [dot] com
#####

library(ggplot2)
library(lemon) # v 0.4.5
library(dplyr)
library(tidyr)

rm(list = ls())

# Read in data and add a label (for panels)
growths = rbind(read.csv('07_analysis_outputs/growth_by_burn_freq_hijuv_original.csv') %>% 
                  mutate(source = 'a) High juvenile survival conditions'),
                read.csv('07_analysis_outputs/growth_by_burn_freq_lojuv_original.csv') %>% 
                  mutate(source = 'b) Low juvenile survival conditions')) %>%
  # Change popultion labels to captalized and relabel two based on reviewer advice
  mutate(site = toupper(site)) %>%
  mutate(site = ifelse(site %in% 'NNWLF', 'NNW', site),
         site = ifelse(site %in% 'RRXDC', 'RDC', site)) %>%
  # Plot only frequncies between 0% and 50%
  filter(freq < 0.6) %>%
  rename(Site = site)

growths %>%  
  # Plot
  ggplot(aes(x = freq, y = exp(mean), group = Site)) +
  geom_segment(x = -1, xend = 1, y = 1, yend = 1, color = 'gray', size = 1) +
  geom_line(size = 1) +
  geom_ribbon(aes(ymin = exp(mean - 2*sd/sqrt(500)), 
                  ymax = exp(mean + 2*sd/sqrt(500)), 
                  fill = Site), alpha = 0.5) +
  geom_point(aes(shape = Site), fill = 'black', size = 6) +
  geom_point(aes(shape = Site, fill = Site), size = 5) +
  geom_text(aes(x = 0, y = 1.005, label = source),
            hjust = 0, vjust = 0, size = 5,
            data = growths %>% distinct(source, .keep_all = TRUE)) +
  guides(shape = guide_legend(ncol = 2, byrow = TRUE),
         color = guide_legend(ncol = 2, byrow = TRUE),
         fill  = guide_legend(ncol = 2, byrow = TRUE)) +
  theme_classic() +
  theme(strip.background = element_blank(),
        strip.text.x = element_blank(),
        panel.background = element_blank(),
        panel.grid = element_blank(),
        axis.ticks.length = unit(0.25, 'cm'),
        axis.title = element_text(size = 15),
        axis.text  = element_text(size = 11),
        legend.key = element_blank(),
        legend.justification = 'left',
        legend.position = c(0.01, 0.15),
        legend.title = element_blank(),
        legend.text = element_text(size = 13)) +
  scale_fill_grey() +
  scale_color_grey() +
  scale_shape_manual(values = 21:25) +
  scale_x_continuous(breaks = (0:2)*0.2 + 0.1, labels = (0:2)*0.2 + 0.1) +
  scale_y_continuous(breaks = c(0.9, 0.95, 1.0), labels = c("0.90", "0.95", "1.00"),
                     minor_breaks = (86:100)/100,
                     limits = c(0.86, 1.01)) +
  xlab('Annual burn probability') +
  ylab(expression(paste('Mean stochastic growth rate, ', bar(lambda[s])))) +
  labs(colour = '', fill = '', shape = '') +
  facet_rep_wrap(~ source, scales = 'free', repeat.tick.labels = 'right') +
  ggsave('09_figure_out/fig_4.png', width = 8, height = 5)

