##### 
# Script for regenerating Figure 5 from Nordstrom et al. 2021
# Figure is the contribution of fire-induced changes in each vital rate
# to the population growth rate.
# Questions: scottwatsonnordsrom [at] gmail [dot] com
#####

# Load packages
library(ggplot2)
library(lemon) 
library(dplyr)
library(tidyr)

rm(list = ls())

# Stylized labels for each vital rate
subscripted =  c(expression("r"["F"]), expression("f"["B"]),
                 expression("f"["F"]), expression("f"["J3"]),
                 expression("s"["B"]), expression("s"["F"]),
                 expression("s"["J3"]), expression("s"["J2"]),
                 expression("s"["J1"]), expression("s"["S"]))

### Main plot code

# Read in data, adding some labels
# (read in two sources separately)
rdecom = rbind(read.csv('07_analysis_outputs/rate_decomposition_output_hijuv_original.csv') %>%
                 mutate(source = 'High juvenile survival conditions',
                        rate = factor(rate, levels = c("r_f", "f_b", "f_f", "f_j3", "s_b", "s_f",
                                                       "s_j3", "s_j2", "s_j1", "s_s"))),
               read.csv('07_analysis_outputs/rate_decomposition_output_lojuv_original.csv') %>%
                 mutate(source = 'Low juvenile survival conditions',
                        rate = factor(rate, levels = c("r_f", "f_b", "f_f", "f_j3", "s_b", "s_f")))) %>%
  # Discard the 20% burn probability output (just clutters plot)
  filter(prob %in% c(0.1, 0.5)) %>%
  mutate(prob = ifelse(prob %in% 0.1, 'Annual Burn Prob. 10%', 'Annual Burn Prob. 50%')) %>%
  # Reformat population names (change labels upon reviewer requests and capitalize)
  mutate(site = toupper(site)) %>%
  mutate(site = ifelse(site %in% 'NNWLF', 'NNW', ifelse(site %in% 'RRXDC', 'RDC', site))) %>%
  rename(Site = site) %>%
  mutate(label = ifelse(grepl('10', prob), 
                        ifelse(grepl('High', source), 'a)', 'b)'),
                        ifelse(grepl('High', source), 'c)', 'd)')))

rdecom %>%
  # Code for plot
  ggplot(aes(x = rate, y = as.factor(prob), group = Site, fill = Site)) +
  # Dividers separating/grouping vital rates
  geom_segment(x = 1.5, xend = 1.5, y = -1, yend = 1) +
  geom_segment(x = 4.5, xend = 4.5, y = -1, yend = 1) +
  geom_segment(x = 2.5, xend = 2.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_segment(x = 3.5, xend = 3.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_segment(x = 5.5, xend = 5.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_segment(x = 6.5, xend = 6.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_segment(x = 7.5, xend = 7.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_segment(x = 8.5, xend = 8.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_segment(x = 9.5, xend = 9.5, y = -1, yend = 1, col = 'gray88', size = 0.25) +
  geom_col(aes(x = rate, y = median), position = position_dodge2(width = 0.5), colour = 'black') +
  # geom_text(aes(x = 1, y = 0.035, hjust = 0.5, vjust = 0,
  #               label = ifelse(grepl('10', prob), ifelse(grepl('High', source), 'a)', 'c)'),
  #                              ifelse(grepl('High', source), 'b)', 'd)')))) +
  geom_label(aes(x = 0.5, y = 0.033,
                 label = paste0(label, ' ', source, ',\n    ', prob)),
             hjust = 0, vjust = 0.5, fill = 'white', size = 3.1,
             label.size = NA, alpha = 0.925,
             data = rdecom %>% distinct(label, .keep_all = TRUE)) +
  scale_fill_grey(start = 0.05, end = 0.95) +
  scale_x_discrete(breaks = waiver(), labels = subscripted) +
  scale_y_continuous(limits = c(-0.05, 0.04), breaks = (-2:1)/40,
                     labels = c("-0.050", "-0.025", "0.000", "0.025")) +
  theme_classic() +
  theme(strip.background = element_blank(),
        strip.text = element_blank(),
        axis.ticks.length = unit(0.25, 'cm'),
        axis.text.x  = element_text(size = 13, vjust = 0),
        axis.title.x = element_text(vjust = -0.85),
        panel.border = element_blank(),
        panel.grid = element_blank(),
        panel.background = element_blank(),
        legend.direction = 'horizontal',
        legend.position = 'bottom',
        axis.title = element_text(size = 15)) +
  # facet_grid(rows = vars(prob), cols = vars(source), scales = 'free') +
  facet_rep_wrap(prob ~ source, 
                 ncol = 2, scales = 'free', 
                 repeat.tick.labels = c('right')) +
                 #repeat.tick.labels = c('right', 'top')) +
  labs(x = 'Vital Rate', 
       y = expression(paste('Contribution to growth rate, ', bar(r[i]))),
       colour = '', fill = '') +
  ggsave('09_figure_out/fig_5.png', width = 8, height = 5)

